<?php

namespace Service;

use Exception\CustomException;
use Teknisa\Model\Geral\Entities\Operador;
use Util\InstanceProvider;
use Util\Utilities;
use Doctrine\ORM\EntityManager;
use \Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Comment
{

    public static $COMMENT_PUBLIC = 'Y';
    public static $COMMENT_NOT_PUBLIC = 'N';
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var Utilities
     */
    private $environment;
    private $emailsFailedToSend = array();

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager  = $entityManager;
        $this->environment    = InstanceProvider::getEnvironment();


    }
    public function saveCommentOnIssuesByProductCallComment($issueId, $description,$public) {
        $connection  = $this->entityManager->getConnection();
        $sendEmail = false;
        try {
            $this->entityManager->beginTransaction();
            /** @var Entities\Task $issue */
            $issue = InstanceProvider::getRepository()->getTask()->find($issueId);
            $reporterId = InstanceProvider::getEnvironment()->getCurrentUserId();
            $commentType = InstanceProvider::getRepository()->getCommentType()->findOneBy(array('control'=>Entities\CommentType::TYPE_COMMON));
            $public = $public == self::$COMMENT_PUBLIC ? self::$COMMENT_PUBLIC : self::$COMMENT_NOT_PUBLIC;
            $comment = InstanceProvider::getFactory()->getComments()->createComments($issue, $description,$commentType,$public);
            $issue->setCurrentTaskStatusIn(Utilities::getEnvironment()->getCurrentTime());
            $this->entityManager->persist($issue);
            $this->entityManager->persist($comment);
            $this->entityManager->flush();
            InstanceProvider::getEnvironment()->setCurrentUserId($reporterId);

            $connection->commit();
            $sendEmail = true;
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        if($sendEmail){
            //$this->sendsMailAdvisingAboutNewIssueComment($issueId, $description,$public == self::$COMMENT_PUBLIC);
        }
        return $comment->getId();
    }

    /****************************************************************************
     * Sends mail advising to all who commented on a specified issue that there
     * is a new comment available.
     * @param $issueId
     * @param $comment
     * @param bool $public
     */
    public function sendsMailAdvisingAboutNewIssueComment($issueId, $comment, $public = true){
        $taskRepository = InstanceProvider::getRepository()->getTask();
        $organizationRepository = InstanceProvider::getRepository()->getOrganization();
        $issueMail = InstanceProvider::getMailIssueMail();

        $commenterId    = $this->environment->getCurrentUserId();
        $emails         = $taskRepository->getRelatedResourcesEmailsByIssueId($issueId, $commenterId);
        /** @var Entities\Task $issue */
        $issue          = $taskRepository->find($issueId);
        $issueName      = $issue->getName();
        $reporterName   = DevOpsUtilities::getResourceNameForSendingMail($issue, Entities\Allocation::TYPE_REPORTER);
        $sponsor        = DevOpsUtilities::getResourceNameForSendingMail($issue, Entities\Allocation::TYPE_SPONSOR);
        $organizationName = $organizationRepository->find($issue->getOrganization())->getName();
        $commenterName  = $this->environment->getCurrentUser()->getName();
        $date           = $this->environment->getCurrentTime()->format(Utilities::FORMAT_DATE_DEFAULT_NO_SECS);
        $issueTitle     = $issueId." - ".$issueName;
        $productName    = $issue->getProject()->getName();
        $mailMessage = "";
        if(!empty($emails)){
            foreach($emails as $email){
                if ($email['EMAIL'] && ($public || strpos($email['EMAIL'],'@teknisa.com') != false) ){
                    $task = $taskRepository->find($issueId);
                    $mailMessage .= $issueMail->buildBodyMessage($task);
                    if(!$issueMail->sendCommentMail($email['EMAIL'], $commenterName, $comment, $date, $issueTitle,
                        $issueName, $reporterName, $sponsor, $issueId, $productName, $organizationName)){
                        array_push($this->emailsFailedToSend,$email['EMAIL']);
                    }
                }
            }
        }
    }



    public function saveCallComment($description, $callRelationship , $callType, $subject = 'Adicionou novo comentário')
    {
        $commentDate ='';
        $intranetIntegrationService = InstanceProvider::getService()->getINTRANETIntegration();
        $connection = $this->entityManager->getConnection();
        try {

            $connection->beginTransaction();
            $createdBy = $this->environment->getCurrentCdoperador();

            /** @var Operador $commenter */
            $commenter = InstanceProvider::getRepository()->getOperador()->findOneBy(array('cdoperador' => $this->environment->getCurrentCdoperador()));

            if($callType == Call::TYPE_ISSUE) {
                $this->saveCommentOnIssuesByProductCallComment($callRelationship, $description, $createdBy);
            } else if($callType == Call::TYPE_RICH_CLIENT) {

                $nrorgId = $this->environment->getCurrentOrganizationId();
                $commentDate = $intranetIntegrationService->saveCommentOnSolicitacaoByProductCallComment($callRelationship, $subject, $description, $createdBy,$nrorgId);

                $comments = InstanceProvider::getRepository()->getCallComments()->getProductCallCommentsFromSolicitacao($callRelationship,null,true);
                if($comments && !empty($comments[1])){
                    $newFilename = $comments[0]['ORDERBY'];
                    $filename = $comments[1]['ORDERBY'];
                    $cdClienteBase = \Util\Utilities::getCdClienteBase();
                    $fileDir = $intranetIntegrationService->basicIntranetFileDir;
                    $folders    = InstanceProvider::getRepository()->getCallAttachment()->getAllAttachmentsFolder($callRelationship,
                        $fileDir, $cdClienteBase, true, $filename);

                    if($folders){
                        foreach($folders as $folder){
                            $newFolder = str_replace($filename, $newFilename, $folder['DIR']);
                            if(file_exists($folder['DIR'])){
                                rename($folder['DIR'], $newFolder);
                            }
                        }
                    }
                }
            }

            $connection->commit();
            $this->sendEmailGep($commentDate,$callRelationship);
            return $commentDate;
        } catch(\Exception $e) {
            $connection->rollBack();
            throw new CustomException('ERR_NOT_POSSIBLE_TO_CREATE_COMMENT', 500, array( 'error' => $e->getMessage()));
        }
    }

    public function sendEmailGep($commentDate,$callRelationship){
//        $commentDateString = $commentDate->format('dmYHis');
//        $emails = InstanceProvider::getRepository()->getSoliservLocal()->getUnsentEmails($commentDateString,$callRelationship);
//        foreach($emails as $i=>$email){
//            $message = $this->getEmailMessage($email);
//            try {
//                $this->encodeAndSend($email["DSEMAILOPER"], "Solicitação ".$email["DSREFESOLI"], $message);
//                $success = "\033[1;32mSucesso";
//                $this->updateEmailEnvProductCallComment($email["CDFILIAL"],$email["NRSOLISERV"], new \DateTime(str_replace("/","-",$email["DTALFASETRAM"])));
//            }catch(\Exception $e){
//                $success = "\033[1;31mFalha\n\033[0mMotivo da falha: \033[1;31m".$e->getMessage()."\033[0m";
//            }
//            $this->showEmailLog($email,$success);
//        }
    }
}
